/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/osinfo.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/psinfo.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


static	pid_t			proxy_refresh_zoom;
static	timer_t			timer_refresh_zoom;


int refresh_zoom( void *data, pid_t pid, void *msg, ushort_t size )
	{

	/* eliminate 'unreferenced' warnings */
	data = data, pid = pid, msg = msg, size = size;	

	/*--- Update the zoom panel ---*/
	do_zoom_redraw();

	/*--- Flush pending proxy ---*/
	while ( Creceive( proxy_refresh_zoom, NULL, 0 ) == proxy_refresh_zoom );

	return Pt_CONTINUE;
	
	}	// refresh_zoom



int
start_drag_pan( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PhRect_t			rect1, rect2;
	PhPoint_t			pos;
	short				drag_flag;
	struct sigevent		sigevnt;
	PhDim_t				step;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Calculate the area that the pan can be dragged ---*/
	PtWidgetExtent( ABW_life_bmp1, &rect2 );
	pos.x = -rect2.ul.x;
	pos.y = -rect2.ul.y;
	PtTranslateRect( &rect2, &pos );
	
	/*--- Function signaled each second to refresh PC date/time ---*/
	proxy_refresh_zoom = qnx_proxy_attach( 0, 0, 0, -1 );
	sigevnt.sigev_signo = -proxy_refresh_zoom;
	timer_refresh_zoom = timer_create( CLOCK_REALTIME, &sigevnt );
	PtAppAddInput( NULL, proxy_refresh_zoom, refresh_zoom, NULL );

	/*--- Start the drag ---*/
	drag_flag  = Ph_TRACK_DRAG;
	drag_flag |= Ph_DRAG_TRACK; // Remove this for outline dragging
	step.w = 8;		// granularity
	step.h = 1;
	PhInitDrag( PtWidgetRid( ABW_life ), drag_flag, PtWidgetExtent( widget, &rect1 ),
		&rect2, cbinfo->event->input_group, 0, 0, &step );

	/*--- Change the cursor while dragging the pan ---*/
	setCursorType( ABW_life, Ph_CURSOR_FINGER );

	return( Pt_CONTINUE );

	}	// start_drag_pan


int
drag_pan_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PhRect_t			*rect;
	PhPoint_t			pos;
	PtArg_t				args[1];
	struct itimerspec	timerSpec;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Compute the new position of the widget ---*/
	rect = PhGetData( cbinfo->event );
	PtTranslateRect( rect, &cbinfo->event->translation );

	/*--- Draw now the widget being dragged at the new position ---*/
	pos.x  = rect->ul.x;
	pos.y  = rect->ul.y;
	PtSetArg( &args[0], Pt_ARG_POS, &pos, 0 );
	PtSetResources( ABW_life_pan, 1, args );
	PtFlush();

	/*--- Update the zoom window ---*/
	zoom_start_x = pos.x;
	zoom_start_y = pos.y;
	set_zoom_xscrollbar( zoom_start_x );
	set_zoom_yscrollbar( zoom_start_y );

	/*--- End of dragging ? ---*/
	if ( cbinfo->event->subtype == Ph_EV_DRAG_COMPLETE )
		{

		/*--- Restore the cursor at the end of dragging the pan ---*/
		setCursorType( ABW_life, Ph_CURSOR_POINTER );

		/*--- Release the timer used for zoom refresh ---*/
		timer_delete( timer_refresh_zoom );
		qnx_proxy_detach( proxy_refresh_zoom );

		/*--- Update the zoom panel ---*/
		do_zoom_redraw();

		/*--- Give back the focus to the zoom window ---*/
		window_focus( ABW_zoom );

		}
	else
		{

		timerSpec.it_value.tv_sec     = 0;				// 0 sec. before initial firing
		timerSpec.it_value.tv_nsec    = 150000000L;
		timerSpec.it_interval.tv_sec  = 0;				// repetitive timer afterwards
		timerSpec.it_interval.tv_nsec = 0;
		timer_settime( timer_refresh_zoom, 0, &timerSpec, NULL );

		}

	return( Pt_CONTINUE );

	}	// drag_pan_cb
