/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <process.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


static void update_config_file( void )
	{
	char		temp[ 100 ];
	PtArg_t		args[ 1 ];
	PhArea_t	*area;
	int			x, y;

	/*--- Open the configuration file ---*/	
	if ( PxConfigOpen( ini_fname, PXCONFIG_WRITE ) == Pt_FALSE )
		return;

	/*--- Windows Size and Position ---*/	
	PxConfigSection( CFG_SECTION_WINSZPOS );

	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( ABW_zoom, 1, args );
	force_coord_into_console( area->pos.x, area->pos.y, &x, &y );
	sprintf( temp, "%d,%d,%d,%d", 
		x, y, area->size.w, area->size.h );
	PxConfigWriteString( CFG_SECTION_WINSZPOS, "Zoom window", PXCONFIG_FMT_STRING, temp );

	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( ABW_life, 1, args );
	force_coord_into_console( area->pos.x, area->pos.y, &x, &y );
	sprintf( temp, "%d,%d,%d,%d", 
		x, y, area->size.w, area->size.h );
	PxConfigWriteString( CFG_SECTION_WINSZPOS, "Life window", PXCONFIG_FMT_STRING, temp );
		
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( ABW_status, 1, args );
	force_coord_into_console( area->pos.x, area->pos.y, &x, &y );
	sprintf( temp, "%d,%d,%d,%d", 
		x, y, area->size.w, area->size.h );
	PxConfigWriteString( CFG_SECTION_WINSZPOS, "Status window", PXCONFIG_FMT_STRING, temp );
		
	/*--- Options ---*/	
	PxConfigSection( CFG_SECTION_OPTIONS );
	PxConfigWriteString( CFG_SECTION_OPTIONS, "Grid", PXCONFIG_FMT_STRING,
		(is_grid) ? "yes" : "no" );
	PxConfigWriteInt( CFG_SECTION_OPTIONS, "Kept", PXCONFIG_FMT_INT_DECIMAL,
		max_nb_gen_kept );
	PxConfigWriteString( CFG_SECTION_OPTIONS, "snp", PXCONFIG_FMT_STRING,
		(is_snp_abort) ? "yes" : "no" );

	/*--- Life Rules ---*/
	PxConfigSection( CFG_SECTION_LRULES );
	PxConfigWriteInt( CFG_SECTION_LRULES, "Births", PXCONFIG_FMT_INT_DECIMAL,
		wrk_lrules.births );
	PxConfigWriteInt( CFG_SECTION_LRULES, "Deaths", PXCONFIG_FMT_INT_DECIMAL,
		wrk_lrules.deaths );
		
	/*--- Close the configuration file ---*/
	PxConfigClose();

	}	// update_config_file


int readandset_cfg_szpos( char *section, PtWidget_t *wgt )
	{
	char		temp[ 100 ];
	PhArea_t	area;
	PtArg_t		args[ 1 ];

	/*--- Open the configuration file ---*/	
	if ( PxConfigOpen( ini_fname, PXCONFIG_READ ) == Pt_FALSE )
		return -1;

	/*--- Windows Size and Position ---*/	
	PxConfigSection( CFG_SECTION_WINSZPOS );
	if ( PxConfigReadString( CFG_SECTION_WINSZPOS, section, "", 
		temp, sizeof( temp ) - 1 ) == NULL )
		{
		PxConfigClose();
		return -1;
		}
	else
		if ( sscanf( temp, "%d,%d,%d,%d", &area.pos.x, &area.pos.y, 
			&area.size.w, &area.size.h ) != 4 )
			{
			PxConfigClose();
			return -1;
			}

	/*--- Close the configuration file ---*/
	PxConfigClose();

	/*--- Set now the original dimension and position of the window ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetResources( wgt, 1, args );
//	PtWindowConsoleSwitch( PtWidgetRid( wgt ) );
	PtConsoleSwitch( 0 );

	return 0;
	
	}	// read_cfg_szpos_zoom


int save_posdim_in_config_file( char *section, PtWidget_t *wgt )
	{
	char		temp[ 100 ];
	PtArg_t		args[ 1 ];
	PhArea_t	*area;
	int			x, y;

	/*--- Open the configuration file ---*/	
	if ( PxConfigOpen( ini_fname, PXCONFIG_WRITE ) == Pt_FALSE )
		return -1;

	/*--- Windows Size and Position ---*/	
	PxConfigSection( CFG_SECTION_WINSZPOS );
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( wgt, 1, args );
	force_coord_into_console( area->pos.x, area->pos.y, &x, &y );
	sprintf( temp, "%d,%d,%d,%d", 
		x, y, area->size.w, area->size.h );
	PxConfigWriteString( CFG_SECTION_WINSZPOS, section, PXCONFIG_FMT_STRING, temp );

	/*--- Close the configuration file ---*/
	PxConfigClose();

	return 0;
	
	}	// save_posdim_in_config_file


int read_cfg_options( void )
	{
	char	temp[ 100 ];
	int		kept;
	int		n;
#ifdef NEVER
	int		avail;
#endif
	int		status;
	int		births, deaths;

	/*--- Open the configuration file ---*/	
	if ( (status = PxConfigOpen( ini_fname, PXCONFIG_READ )) == Pt_FALSE )
		if ( PxConfigOpen( ini_fname, PXCONFIG_WRITE ) == Pt_FALSE )
			return -1;

	/*--- Options: grid ---*/	
	PxConfigSection( CFG_SECTION_OPTIONS );
	PxConfigReadString( CFG_SECTION_OPTIONS, "Grid", "yes", temp, sizeof( temp ) - 1 );
	ApModifyItemState( &options_menu, 
		(!stricmp( temp, "yes" )) ? AB_ITEM_SET : AB_ITEM_NORMAL,
		ABN_options_grid, NULL );
	is_grid = (!stricmp( temp, "yes" )) ? TRUE : FALSE;

	PxConfigReadInt( CFG_SECTION_OPTIONS, "Kept", MAXNBGENKEPT_DEFAULT,
		&kept );

	PxConfigReadString( CFG_SECTION_OPTIONS, "snp", "yes", temp, sizeof( temp ) - 1 );
	is_snp_abort = (!stricmp( temp, "yes" )) ? TRUE : FALSE;

	/*--- Alloc space for generations kept ---*/
#ifdef NEVER
	avail = _memmax() * 1024;
	if ( ((kept * FIELD_SIZE * 110) / 100) >= avail )
		kept = ((avail * 100) / 110) / FIELD_SIZE;
#endif
	if ( kept != max_nb_gen_kept )
		{

		free( gen_kept );
		free( gen_kept_base );
		free( gen_kept_popnb );
		
		max_nb_gen_kept = kept;
		gen_kept_popnb = app_calloc( max_nb_gen_kept, sizeof( int ) );
		gen_kept = app_calloc( max_nb_gen_kept, sizeof( UBYTE * ) );
		gen_kept_base = app_calloc( max_nb_gen_kept, FIELD_SIZE );
		for ( n = 0; n < max_nb_gen_kept; n++ )
			gen_kept[ n ] = &gen_kept_base[ FIELD_SIZE * n ];
		gen_kept_nb = 0;

		}	// if

	/*--- Life Rules ---*/
	PxConfigSection( CFG_SECTION_LRULES );
	PxConfigReadInt( CFG_SECTION_LRULES, "Births", STD_BIRTHS, &births );
	wrk_lrules.births = births;
	PxConfigReadInt( CFG_SECTION_LRULES, "Deaths", STD_DEATHS, &deaths );
	wrk_lrules.deaths = deaths;

	/*--- Show which rules (status window title) ---*/
	show_which_rules();

	/*--- Close the configuration file ---*/
	PxConfigClose();
	return status;
	
	}	// read_cfg_options


int
file_exit( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Create the configuration file ---*/
	update_config_file();

	/*--- Destroy others windows ---*/
	if ( ABW_stats )
		PtDestroyWidget( ABW_stats );
	if ( ABW_seek )
		PtDestroyWidget( ABW_seek );
	if ( ABW_settings )
		PtDestroyWidget( ABW_settings );
	if ( ABW_rules )
		PtDestroyWidget( ABW_rules );
	if ( ABW_about )
		PtDestroyWidget( ABW_about );
	if ( ABW_lrules )
		PtDestroyWidget( ABW_lrules );
	if ( ABW_graph )
		PtDestroyWidget( ABW_graph );
	if ( ABW_print )
		PtDestroyWidget( ABW_print );
	PtDestroyWidget( ABW_zoom );		/*--- To enable destroyed cb to be called ---*/

	/*--- Des-allocate shared memory for fast draw of life ---*/
	PgShmemDestroy( fast_field );

	/*--- Now exit the application ---*/
	exit( 0 );

	return( Pt_CONTINUE );

	}	// file_exit
