/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


int
file_load( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
static	PtFileSelectionInfo_t	infos;
static	BOOL					first = TRUE;
		char					root_dir[ 200], fname[ 300 ];
		FILE					*fp;
		char					_node[ _MAX_NODE + 1 ], _dir[ _MAX_DIR + 1 ],
								_fname[ _MAX_FNAME + 1 ], _ext[ _MAX_EXT + 1 ];
		LIFE_INFOS				life_infos;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Dont' do while computing next generations (Shift-F2) ---*/
	if ( is_comp_generations )
		return( Pt_CONTINUE );

	/*--- Initialize infos results ---*/
	if ( first )
		{
		memset( &infos, 0, sizeof( infos ) );
		first = FALSE;
		}

	/*--- File dialog selection ---*/
	getcwd( root_dir, sizeof( root_dir ) - 1 );
	if ( PtFileSelection( NULL, NULL, "Load a saved life", root_dir, "*.life",
		"&Load", "&Cancel", "ndk", &infos,
		Pt_FSDIALOG_NO_FCHECK | Pt_FSDIALOG_NO_UP_BUTTON ) == -1 )
		return( Pt_CONTINUE );

	/*--- Cancel ? ---*/
	if ( infos.ret == Pt_FSDIALOG_BTN2 )
		return( Pt_CONTINUE );

	/*--- Open the file ---*/
	if ( (strlen( infos.fspec ) >= 2) && (infos.fspec[ 0 ] == '*') )
		sprintf( fname, "%s.%s", infos.path, &infos.fspec[ 1 ] );
	else
		sprintf( fname, "%s%s", infos.path, infos.fspec );
	if ( (fp = fopen( infos.path, "r" )) == NULL )	
		return( Pt_CONTINUE );

	/*--- Is-it a 'life' file ? ---*/
	if ( filelength( fileno( fp ) ) != (sizeof( LIFE_INFOS ) + FIELD_SIZE) )
		return
			fclose( fp),
			message( "Load a life file", "%s\nThis is not a life file !", fname, NULL ),
			Pt_CONTINUE;

	/*--- Life field: Reset informations (Generation, Population) ---*/
	setString( ABW_status_gen, "%d", Generation = 0 );
	setString( ABW_status_pop, "%d", Population = 0 );
	setString( ABW_status_birth, "" );
	setString( ABW_status_death, "" );

	/*--- Clear all celld of the bitmaps #1 and #2 ---*/
	clear_field();

	/*--- Optimization ---*/
	opt_x1 = 1;
	opt_x2 = FIELD_WIDTH8 - 2;
	opt_y1 = 1;
	opt_y2 = FIELD_HEIGHT - 2;

	/*--- Load the informations relative to this generation ---*/
	/*--- then Load the life ---*/
	if ( fread( &life_infos, 1, sizeof( LIFE_INFOS ), fp ) != sizeof( LIFE_INFOS ) ||
		 fread( bitmap_to_use(), 1, FIELD_SIZE, fp ) != FIELD_SIZE )
		return
			message( "Load a life file", "%s\nError %d loading file.", fname, errno, NULL ),
			fclose( fp ),
			Pt_CONTINUE;
	fclose( fp );

	/*--- Set the current directory ---*/
	_splitpath( infos.path, _node, _dir, _fname, _ext );
	chdir( _dir );

	/*--- Show which rules (status window title) ---*/
	memmove( &wrk_lrules, &life_infos.lrules, sizeof( LRULES ) );
	show_which_rules();

	/*--- Center the pan zoom into the life field ---*/
	zoom_start_x = MULT8( (FIELD_WIDTH  - zoomed_cells_width_nb) / 2) ;
	set_zoom_xscrollbar( zoom_start_x );
	zoom_start_y = (FIELD_HEIGHT - zoomed_cells_height_nb) / 2;
	set_zoom_yscrollbar( zoom_start_y );
	resize_pan();

	/*--- Show the current bitmap ---*/
	damage_current_bitmap();
	show_life( 1 );

	/*--- Redraw the zoom area ---*/
	do_zoom_redraw();

	/*--- Count the number of cells inside the field ---*/
	setString( ABW_status_pop, "%d", 
		Population = count_nb_cells( bitmap_to_use() ) );

	/*--- Field life: Clear stable infos ---*/
	PtUnrealizeWidget( ABW_status_stable_label );
	PtUnrealizeWidget( ABW_status_stable_period );

	/*--- Generations kept to see if stability ---*/
	gen_kept_nb = 0;
	is_stability = FALSE;

	/*--- Initialize statistics and show-it (if window is visible) ---*/
	memset( &gstats, 0, sizeof( STATS ) );
	show_stats();

	/*--- Enable/Disable option 'Save' of the menu ---*/
	ApModifyItemState( &file_menu, 
		(Population > 0) ? AB_ITEM_NORMAL : AB_ITEM_DIM, 
		ABN_file_save, ABN_file_save_text, ABN_file_print,
		NULL );
	
	/*--- Disable option 'Previous generation' of the menu ---*/
	ApModifyItemState( &life_menu, 
		AB_ITEM_DIM, 
		ABN_life_prev, ABN_life_restoregen0,
		NULL );
	
	return( Pt_CONTINUE );

	}	// file_load
