/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


void graph_redraw( void )
	{
	GRAPHDATA		*p;
	int				gen;
	short			max_pop;
	PtArg_t			args[ 1 ];
	PhPoint_t		*points, *q;
	USHORT			height, width;
	int				nbirths, ndeaths, nsurviv;
	double			scale_factor;

	/*--- Allocate space for points ---*/
	points = app_calloc( Generation+2, sizeof( PhPoint_t ) );

	/*--- Compute scale ---*/
	nbirths = ndeaths = nsurviv = 0;
	for ( max_pop = 0, p = &graphdata[ 1 ], gen = 0; gen < Generation; gen++, p++ )
		{
		if ( p->pop > max_pop )
			max_pop = p->pop;
		if ( (p->births > p->deaths) && (p->births > p->surviv) )
			nbirths++;
		else if ( (p->deaths > p->births) && (p->deaths > p->surviv) )
			ndeaths++;
		else if ( (p->surviv > p->births) && (p->surviv > p->deaths) )
			nsurviv++;
		}

	/*--- Change order ---*/
	PtWidgetToFront( ABW_graph_pop );
	if ( (nsurviv > nbirths) && (nsurviv > ndeaths) )
		{
		PtWidgetToFront( ABW_graph_surviv );
		if ( nbirths > ndeaths )
			{
			PtWidgetToFront( ABW_graph_births );
			PtWidgetToFront( ABW_graph_deaths );
			}
		else
			{
			PtWidgetToFront( ABW_graph_deaths );
			PtWidgetToFront( ABW_graph_births );
			}
		}
	else if ( (nbirths > nsurviv) && (nbirths > ndeaths) )
		{
		PtWidgetToFront( ABW_graph_births );
		if ( nsurviv > ndeaths )
			{
			PtWidgetToFront( ABW_graph_surviv );
			PtWidgetToFront( ABW_graph_deaths );
			}
		else
			{
			PtWidgetToFront( ABW_graph_deaths );
			PtWidgetToFront( ABW_graph_surviv );
			}
		}
	else if ( (ndeaths > nsurviv) && (ndeaths > nbirths) )
		{
		PtWidgetToFront( ABW_graph_deaths );
		if ( nsurviv > nbirths )
			{
			PtWidgetToFront( ABW_graph_surviv );
			PtWidgetToFront( ABW_graph_births );
			}
		else
			{
			PtWidgetToFront( ABW_graph_births );
			PtWidgetToFront( ABW_graph_surviv );
			}
		}

	/*--- Scale factor ---*/
	getDim( ABW_graph_pane, &width, &height );
	if ( (Generation == 0) )
		scale_factor = 1;
	else
		scale_factor = ((double)width / (double)Generation);

	/*--- Prepare data to be shown ---*/
	q = points;
	q->x = 0;
	q->y = height;
	for ( p = &graphdata[ 1 ], q++, gen = 0; gen < Generation; gen++, p++, q++ )
		{
		q->x = (gen + 1) * scale_factor;
		q->y = height - ((p->pop * height) / max_pop);
		}
	q->x = (Generation + 1) * scale_factor;
	q->y = height;
	PtSetArg( &args[ 0 ], Pt_ARG_POINTS, points, Generation+2 );
	PtSetResources( ABW_graph_pop, 1, args );

	/*--- Prepare data to be shown ---*/
	q = points;
	q->x = 0;
	q->y = height;
	for ( p = &graphdata[ 1 ], q++, gen = 0; gen < Generation; gen++, p++, q++ )
		{
		q->x = (gen + 1) * scale_factor;
		q->y = height - ((p->births * height) / max_pop);
		}
	q->x = (Generation + 1) * scale_factor;
	q->y = height;
	PtSetArg( &args[ 0 ], Pt_ARG_POINTS, points, Generation+2 );
	PtSetResources( ABW_graph_births, 1, args );

	/*--- Prepare data to be shown ---*/
	q = points;
	q->x = 0;
	q->y = height;
	for ( p = &graphdata[ 1 ], q++, gen = 0; gen < Generation; gen++, p++, q++ )
		{
		q->x = (gen + 1) * scale_factor;
		q->y = height - ((p->deaths * height) / max_pop);
		}
	q->x = (Generation + 1) * scale_factor;
	q->y = height;
	PtSetArg( &args[ 0 ], Pt_ARG_POINTS, points, Generation+2 );
	PtSetResources( ABW_graph_deaths, 1, args );

	/*--- Prepare data to be shown ---*/
	q = points;
	q->x = 0;
	q->y = height;
	for ( p = &graphdata[ 1 ], q++, gen = 0; gen < Generation; gen++, p++, q++ )
		{
		q->x = (gen + 1) * scale_factor;
		q->y = height - ((p->surviv * height) / max_pop);
		}
	q->x = (Generation + 1) * scale_factor;
	q->y = height;
	PtSetArg( &args[ 0 ], Pt_ARG_POINTS, points, Generation+2 );
	PtSetResources( ABW_graph_surviv, 1, args );

	/*--- Ok ---*/
	free( points );

	}	// graph_redraw


void new_graph_data( const int gen, const USHORT pop, const USHORT births,
					 const USHORT deaths, const USHORT surviv )
	{
	GRAPHDATA	*entry;

	/*--- Allocate space ? ---*/	
	if ( gen == graphdata_allcd )
		{
		graphdata_allcd += GRAPHDATA_ALLCD_SZ;
		app_realloc( &graphdata, sizeof( GRAPHDATA ) * graphdata_allcd );
		}

	/*--- Fill the new entry ---*/
	entry = &graphdata[ gen ];
	entry->pop    = pop;
	entry->births = births;
	entry->deaths = deaths;
	entry->surviv = surviv;

	/*--- Update the graphic display ? ---*/
	if ( (ABW_graph != NULL) && (!is_comp_generations) )
		graph_redraw();	
	
	}	// new_graph_data


static void do_graph_resize( void )

	{
	PhArea_t			*graph_area, *old_area, new_area;
	PtArg_t				args[ 1 ];
	PhDim_t				*dim;
	int					total_width;
	unsigned short		spacing;
	PtWidget_t			*curr;
	int					ypos;

	/*--- Get the size of the window ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &graph_area, 0 );
	PtGetResources( ABW_graph, 1, args );

	/*--- Move the graph area pane ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
	PtGetResources( ABW_graph_pane, 1, args );
	memmove( &new_area, old_area, sizeof( PhArea_t ) );
	new_area.size.w = (graph_area->size.w - old_area->pos.x*2 );
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
	PtSetResources( ABW_graph_pane, 1, args );

	/*--- Move the graph area pane ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
	PtGetResources( ABW_graph_show_pane, 1, args );
	memmove( &new_area, old_area, sizeof( PhArea_t ) );
	new_area.size.w = (graph_area->size.w - old_area->pos.x*2 );
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
	PtSetResources( ABW_graph_show_pane, 1, args );

	/*--- Move infos pane to the bottom of the window ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
	PtGetResources( ABW_graph, 1, args );
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
	PtGetResources( ABW_graph_show_pane, 1, args );
	memmove( &new_area, old_area, sizeof( PhArea_t ) );
	new_area.pos.y = (dim->h - old_area->size.h - 8);
	ypos = new_area.pos.y;
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
	PtSetResources( ABW_graph_show_pane, 1, args );

	/*--- Extend the graph pane in height ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
	PtGetResources( ABW_graph_pane, 1, args );
	memmove( &new_area, old_area, sizeof( PhArea_t ) );
	new_area.size.h = (ypos - 11);
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
	PtSetResources( ABW_graph_pane, 1, args );

	/*--- Change the space btw each button ---*/
	curr = ABW_graph_show_pane;
	total_width = 0;
	while ( (curr = PtWidgetFamily( ABW_graph_show_pane, curr )) != NULL )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
		PtGetResources( curr, 1, args );
		if ( PtWidgetIsClass( curr, PtToggleButton ) )
			total_width += dim->w;
		}	// while
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
	PtGetResources( ABW_graph_show_pane, 1, args );
	spacing = (dim->w - 6 - total_width) / 3;
	PtSetArg( &args[ 0 ], Pt_ARG_GROUP_SPACING, spacing, 0 );
	PtSetResources( ABW_graph_show, 1, args );

	/*--- Change the space btw each button ---*/
	curr = ABW_graph_infos_grp;
	total_width = 0;
	while ( (curr = PtWidgetFamily( ABW_graph_infos_grp, curr )) != NULL )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
		PtGetResources( curr, 1, args );
		if ( PtWidgetIsClass( curr, PtGroup ) )
			total_width += dim->w;
		}	// while
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
	PtGetResources( ABW_graph_show_pane, 1, args );
	spacing = (dim->w - 6 - total_width) / 4;
	PtSetArg( &args[ 0 ], Pt_ARG_GROUP_SPACING, spacing, 0 );
	PtSetResources( ABW_graph_infos_grp, 1, args );

	/*--- Redraw now the graph ---*/
	graph_redraw();

	}	// do_graph_resize


int
graph_resize( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Compute the difference and do the resize ---*/
	do_graph_resize( );

	return( Pt_CONTINUE );

	}	// graph_resize


int
graph_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Set the Position/Size of the window if given in the .ini file ---*/
	readandset_cfg_szpos( "Graph window", ABW_graph );

	return( Pt_CONTINUE );

	}	// graph_setup


int
graph_destroyed( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Save position and dimension of this window into the config (life.ini) file ---*/
	save_posdim_in_config_file( "Graph window", ABW_graph );

	/*--- Dialog not mounted ---*/
	ABW_graph = NULL;

	return( Pt_CONTINUE );

	}	// graph_destroyed


int
graph_show_pop( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if ( isBtnSet( widget ) )
		PtRealizeWidget( ABW_graph_pop );
	else
		PtUnrealizeWidget( ABW_graph_pop );

	return( Pt_CONTINUE );

	}	// graph_show_pop


int
graph_show_births( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if ( isBtnSet( widget ) )
		PtRealizeWidget( ABW_graph_births );
	else
		PtUnrealizeWidget( ABW_graph_births );

	return( Pt_CONTINUE );

	}	// graph_show_births


int
graph_show_deaths( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if ( isBtnSet( widget ) )
		PtRealizeWidget( ABW_graph_deaths );
	else
		PtUnrealizeWidget( ABW_graph_deaths );

	return( Pt_CONTINUE );

	}	// graph_show_deaths


int
graph_show_surviv( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if ( isBtnSet( widget ) )
		PtRealizeWidget( ABW_graph_surviv );
	else
		PtUnrealizeWidget( ABW_graph_surviv );

	return( Pt_CONTINUE );

	}	// graph_show_surviv


int
graph_opening( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	do_graph_resize();

	return( Pt_CONTINUE );

	}	// graph_opening


int
graphs_infos_filter( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PhRect_t	*rect;
	int			x, y;
	USHORT		height, width;
	double		scale_factor;
	int			gen;
	GRAPHDATA	*data;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Nothing to do ? ---*/
	if ( Generation == 0 )
		return( Pt_CONTINUE );

	/*--- Get the position of the pointer ---*/
	rect = PhGetRects( cbinfo->event );
	x = rect->ul.x;
	y = rect->ul.y;

	/*--- Scale factor ---*/
	getDim( ABW_graph_pane, &width, &height );
	scale_factor = ((double)width / (double)Generation);

	/*--- Which generation where is the pointer ? ---*/
	gen = (int) ((double)(x - 1) / scale_factor) + 1;
	setString( ABW_graph_infos_gen, "%d", gen );
	data = &graphdata[ gen ];
	setString( ABW_graph_infos_pop, "%d", data->pop );
	#define SHOW( wgt, var ) \
		if ( var == 0 ) \
			PtUnrealizeWidget( wgt ); \
		else \
			{ \
			setString( wgt, "%d", var ); \			
			PtRealizeWidget( wgt ); \
			}
	SHOW( ABW_graph_infos_births, data->births )
	SHOW( ABW_graph_infos_deaths, data->deaths )
	SHOW( ABW_graph_infos_surviv, data->surviv )
	#undef SHOW

	return( Pt_CONTINUE );

	}	// graphs_infos_filter
