#ifndef COMMON
	#define COMMON extern
#endif

// Version number of software
#define VERSION_NB		0x010001		// version 1.0.A
#define VERSION_DATE	0x00050D		// yymmdd

// Options
#define LOGTOTMP	1
#define SIMULATION	1

// Use assembler functions ?
//		0 not at all
//		1 only the compute life is assembler coded
//		2 some others functions too
// NOTE: CURRENTLY, ONLY VALUE 1 IS WORKING
#define USE_ASM 1

// Common things
typedef signed char BOOL;
typedef short int SHORT;
typedef short unsigned int USHORT;
typedef signed char BYTE;
typedef unsigned char UBYTE;
#define PUBLIC
#define FALSE 0
#define TRUE -1

// Size of the life field (hard-coded...)
#define FIELD_WIDTH		800
#define FIELD_WIDTH8	100
#define FIELD_HEIGHT	600
#define FIELD_SIZE		(FIELD_HEIGHT * FIELD_WIDTH8)
#define MULT8( N )		(((N)/8)*8)

// Size of a zoomed cell
#define CELLZ_HEIGHT	10
#define CELLZ_WIDTH		9
#define CELLZ_XGAP2		2
#define CELLZ_XGAP		4
#define CELLZ_YGAP2		4
#define CELLZ_YGAP		4

// Configuration file
#define CFG_FILENAME			"life.ini"
#define CFG_SECTION_WINSZPOS	"Windows Size and Position"
#define	CFG_SECTION_OPTIONS		"Options"
#define	CFG_SECTION_SEEK		"Seek"
#define	CFG_SECTION_LRULES		"Life Rules"

// Zoom field
COMMON	int			zoomed_cells_width_nb, 			// Nb of zoomed cells in width
					zoomed_cells_height_nb;			// Nb of zoomed cells in height

// Generation and Population
COMMON	long		Generation;
COMMON	long		Population;
COMMON	long		births, deaths, surviv;

// Starting (top left corner) of the zoom window into the field life
COMMON	int			zoom_start_x;
COMMON	int			zoom_start_y;

#pragma aux compute_gen parm [esi] [edi];
extern void compute_gen( const UBYTE *src, const UBYTE *dst );

#pragma aux bitmap_to_bytenibble parm [esi] [edi];
extern void bitmap_to_bytenibble( const UBYTE *src, const UBYTE *dst );

#if USE_ASM == 2
#pragma aux genmove parm [edi] [esi];
extern void genmove( UBYTE *dst, const UBYTE *src );
#define GENMOVE( a, b ) genmove( a, b )
#else
#define GENMOVE( a, b ) memmove( a, b, FIELD_SIZE )
#endif

#if USE_ASM == 2
#pragma aux gencopy parm [edi] [esi] [ecx];
extern void gencopy( UBYTE *dst, const UBYTE *src, const int sz );
#else
#endif

#if USE_ASM == 2
#pragma aux gencmp parm [edi] [esi] value [eax];
extern int gencmp( UBYTE *dst, const UBYTE *src );
#else
#endif

// Life field (shared memory)
COMMON	UBYTE		*field1, *field2;
COMMON	UBYTE		*fast_field;

// Optimization
COMMON	int			opt_x1, opt_x2;
COMMON	int			opt_y1, opt_y2;

// Keep trace of the options flag (menu)
COMMON	BOOL		is_grid;
COMMON	BOOL		is_snp_abort;

// Set to TRUE while computing next generations (Shift-F2)
COMMON	BOOL		is_comp_generations;

// Set to TRUE when the user clicks on button 'Stop'
COMMON	BOOL		end_comp_generations;

// Generations kept to see if stability
COMMON	int			*gen_kept_popnb;
COMMON	UBYTE		*gen_kept_base;
COMMON	int			max_nb_gen_kept;
#define	MAXNBGENKEPT_DEFAULT	30
COMMON	UBYTE		**gen_kept;
COMMON	int			gen_kept_nb;
COMMON	BOOL		is_stability;
COMMON	int			stability_period;

// In case of Alt-F2 (Automatic, Next Generations), time elapsed
COMMON	time_t		nextgens_timeelps;

// Generation kept in case of automatic/manual seeking
COMMON	UBYTE		*gen_start_kept_seek;
COMMON	UBYTE		*gen_best_kept_seek;

// Generation 0 kept
COMMON	UBYTE		*gen0_kept;

// Statistics
typedef struct
	{
	unsigned int	total_pop;
	unsigned int	total_births;
	unsigned int	total_deaths;
	unsigned int	total_surviv;
	unsigned int	total_ctime;			// compute time in msec
	unsigned int	total_dtime;			// draw time in msec
	unsigned int	total_ttime;			// total time in msec
	}	STATS;
COMMON	STATS		gstats;

// Graphics
typedef struct
	{
	unsigned int	pop;
	unsigned int	births;
	unsigned int	deaths;
	unsigned int	surviv;
	}	GRAPHDATA;
COMMON	GRAPHDATA	*graphdata;
COMMON	int			graphdata_allcd;		// Nb of cells of the array allocated
#define	GRAPHDATA_ALLCD_SZ	16

// Life Rules Structure
typedef struct
	{
	UBYTE	births;							// 8 bits
	UBYTE	births_prob[ 8 ];				// Probabilities [0-100], or 0xFF if none
	UBYTE	deaths;							// 8 bits
	UBYTE	deaths_prob[ 8 ];				// Probabilities [0-100], or 0xFF if none
	UBYTE	surviv;							// 8 bits
	UBYTE	surviv_prob[ 8 ];				// Probabilities [0-100], or 0xFF if none
	}	LRULES;
COMMON	LRULES	wrk_lrules;					// Common lrules
COMMON	int		is_std_rules;				// Are we using standard rules ?
#define	STD_BIRTHS	0x04					/* 0000 0100 */
#define STD_DEATHS	0xF9					/* 1111 1001 */

// Life Infos Structure (Save/Load), prior to the life field (100x600 bytes)
#define	MAX_COMMENT	255
typedef struct
	{
	long	software_version;				// See VERSION_NB
	int		x1, y1, x2, y2;					// Position of the generation into the field
	char	comment[ MAX_COMMENT + 1];		// Comment associated to this generation
	LRULES	lrules;							// Rules to use on the generation
	}	LIFE_INFOS;

/*--- .ini filename ---*/
COMMON	char	ini_fname[ PATH_MAX+30];

#undef COMMON

