/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <limits.h>
#include <pwd.h>
#include <stdarg.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#define COMMON
#include "life.h"
#include "abimport.h"
#include "proto.h"

/* Application Options string */
const char ApOptions[] =
	AB_OPTIONS ""; /* Add your options in the "" */


/*--- y is height ---*/
void set_zoom_yscrollbar( const int new_pos )
	{
	PtArg_t		args[ 2 ];
	int			new_max;

	/*--- Set the scrollbar ---*/
	new_max = FIELD_HEIGHT - zoomed_cells_height_nb;
	PtSetArg( &args[ 0 ], Pt_ARG_MAXIMUM, new_max, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_SCROLL_POSITION, new_pos, 0 );
	PtSetResources( ABW_zoom_area_yscrollbar, 2, args );

	}	// set_zoom_yscrollbar


/*--- x is width ---*/
void set_zoom_xscrollbar( const int new_pos )
	{
	PtArg_t		args[ 2 ];
	int			new_max;

	/*--- Set the scrollbar ---*/
	new_max = FIELD_WIDTH - zoomed_cells_width_nb;
	PtSetArg( &args[ 0 ], Pt_ARG_MAXIMUM, new_max, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_SCROLL_POSITION, new_pos, 0 );
	PtSetResources( ABW_zoom_area_xscrollbar, 2, args );

	}	// set_zoom_xscrollbar


/*--- Set the grey rectangle inside the field of life ---*/
void resize_pan( void )
	{
	PhArea_t	area;
	PtArg_t		args[ 1 ];

	area.pos.x  = zoom_start_x;
	area.pos.y  = zoom_start_y;
	area.size.w = zoomed_cells_width_nb;
	area.size.h = zoomed_cells_height_nb;

	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetResources( ABW_life_pan, 1, args );

	PtWidgetToBack( ABW_life_pan );

	}	// resize_pan


/*--- The bitmap to use ---*/
UBYTE * bitmap_to_use( void )
	{
	return ( (Generation % 2) ? field2 : field1 );
	}	// bitmap_to_use


/*--- Clear the life field ---*/
void clear_field( void )
	{
	memset( field1, 0, FIELD_SIZE );
	memset( field2, 0, FIELD_SIZE );
	}	// clear_field


void show_life( const int no )
	{
	if ( no == 1 )
		{
		PtUnrealizeWidget( ABW_life_bmp2 );
		PtRealizeWidget( ABW_life_bmp1 );
		}
	else if ( no == 2 )
		{
		PtUnrealizeWidget( ABW_life_bmp1 );
		PtRealizeWidget( ABW_life_bmp2 );
		}
	}	// show_life


int count_nb_cells( const UBYTE *bitmap )
	{
		int		cells_nb;
		int		row, col, k;
		int		w;
static	UBYTE	masks[ 8 ] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
		const UBYTE	*p, *q;
	
	cells_nb = 0;
	p = bitmap;
	for ( row = 0; row < FIELD_HEIGHT; row++ )
		for ( col = 0; col < FIELD_WIDTH8; col++ )
			for ( w = *p++, q = masks, k = 8; k; k-- )
				if ( w & *q++ )
					cells_nb++;
		
	return cells_nb;
	
	}	// count_nb_cells


void find_pos_gen( const UBYTE *bitmap, int *x1, int *x2, int *y1, int *y2 )
	{
		int		row, col, k, rcol;
		int		w;
static	UBYTE	masks[ 8 ] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
		const UBYTE	*p, *q;
	
	/*--- Initialize the position ---*/
	*x1 = *y1 = INT_MAX;
	*x2 = *y2 = INT_MIN;

	/*--- Check all the field ---*/
	for ( p = bitmap, row = 0; row < FIELD_HEIGHT; row++ )
		for ( col = 0; col < FIELD_WIDTH8; col++ )
			for ( w = *p++, q = masks, k = 0; k < 8; k++ )
				if ( w & *q++ )
					{
					if ( row < *x1 )	*x1 = row;
					if ( row > *x2 )	*x2 = row;
					rcol = (col << 3) + k;
					if ( rcol < *y1 )	*y1 = rcol;
					if ( rcol > *y2 )	*y2 = rcol;
					}
		
	}	// find_pos_gen


#if LOGTOTMP==1
void slog( const char *fmt, ... )
	{
	va_list			ap;
	char			msg[ 300 ];
	FILE			*fp;
	time_t			now;
	struct tm		*tm;

	/*--- Develop text to log ---*/
	va_start( ap, fmt );
	vsprintf( msg, fmt, ap );
	va_end( ap );

	/*--- Update log file ---*/
	time( &now );
	tm = localtime( &now );
	if ( (fp = fopen( "/tmp/log.txt", "a" )) != NULL )
		{
		fprintf( fp, "%02d/%02d %02d:%02d:%02d [life] [%s] %s\n",
			tm->tm_mday, tm->tm_mon+1,
			tm->tm_hour, tm->tm_min, tm->tm_sec,
			getlogin(),
			msg );
		fclose( fp );
		}

	}	// slog
#endif


int
life_init( int argc, char *argv[] )

	{
	int				n;
	struct passwd	*pw;
PhCursorInfo_t	infos;

	/* eliminate 'unreferenced' warnings */
	argc = argc, argv = argv;

#if LOGTOTMP==1
	slog( "start" );
#endif


	PhQueryCursor( 1, &infos );


	/*--- 16K buffer for Photon ---*/
	PgSetDrawBufferSize( 16*1024 );

	/*--- Allocate memory for life ---*/
	field1 = app_malloc( FIELD_SIZE  );
	field2 = app_malloc( FIELD_SIZE  );
	clear_field();

	/*--- Allocate shared memory for life ---*/
	if ( (fast_field = PgShmemCreate( (FIELD_WIDTH * FIELD_HEIGHT) / 2, "life_img" )) == NULL )
		{
		int	ref;
		if ( (ref = shm_open( "life_img", O_RDWR, S_IRUSR | S_IWUSR )) == -1 )
			exit( -1 );
		if ( (fast_field = mmap( 0, (FIELD_WIDTH * FIELD_HEIGHT) / 2, 
			PROT_READ | PROT_WRITE, MAP_SHARED, ref, 0 )) == (void*) -1 )
			exit( -1 );
		}

	/*--- Generation and Population ---*/
	Generation = 0;
	Population = 0;
	births = deaths = surviv = 0;

	/*--- Set to TRUE while computing next generations (Shift-F2) ---*/
	is_comp_generations = FALSE;

	/*--- Keep trace of the options flag (menu) ---*/
	// Should read the state inside the menu. I don't know how to do that...
	is_grid			= TRUE;
	is_snp_abort	= TRUE;

	/*--- Optimization ---*/
	opt_x1 = 1;
	opt_x2 = FIELD_WIDTH8 - 2;
	opt_y1 = 1;
	opt_y2 = FIELD_HEIGHT - 2;

	/*--- Dialogs not yet mounted ---*/
	ABW_stats    =
	ABW_seek     =
	ABW_settings =
	ABW_rules	 =
	ABW_about	 =
	ABW_lrules	 =
	ABW_graph	 =
	ABW_print	 = NULL;

	/*--- .ini filename ---*/
	strcpy( ini_fname, "" );
	if ( getlogin() && ((pw = getpwnam( getlogin() )) != NULL) )
		{
		char	p[ PATH_MAX+1 ];
		int		l;
		strcpy( p, pw->pw_dir );
		if ( ((l=strlen(p)) >= 1) && (p[l-1]=='/') )
			p[l-1] = 0;
		sprintf( ini_fname, "%s/.photon/%s", p, CFG_FILENAME );
		}

	/*--- Generations kept to see if ---*/
	max_nb_gen_kept = MAXNBGENKEPT_DEFAULT;
	gen_kept_popnb = app_calloc( max_nb_gen_kept, sizeof( int ) );
	gen_kept = app_calloc( max_nb_gen_kept, sizeof( UBYTE * ) );
	gen_kept_base = app_calloc( max_nb_gen_kept, FIELD_SIZE );
	for ( n = 0; n < max_nb_gen_kept; n++ )
		gen_kept[ n ] = &gen_kept_base[ FIELD_SIZE * n ];
	gen_kept_nb = 0;
	is_stability = FALSE;

	/*--- Generation kept in case of automatic/manual seeking ---*/
	gen_start_kept_seek  = app_malloc( FIELD_SIZE );
	gen_best_kept_seek   = app_malloc( FIELD_SIZE );

	/*--- Generation 0 kept ---*/
	gen0_kept = app_malloc( FIELD_SIZE );

	/*--- Graph data ---*/
	graphdata = app_malloc( sizeof( GRAPHDATA ) * GRAPHDATA_ALLCD_SZ );
	graphdata_allcd = GRAPHDATA_ALLCD_SZ;

	/*--- Open the 'status' window ---*/
	ApCreateModule( ABM_status, NULL, NULL );

	/*--- Create life standard rules ---*/
	create_std_rule( &wrk_lrules );

	/*--- Open the 'field of life' window ---*/
	ApCreateModule( ABM_life, NULL, NULL );

	/*--- Open the 'zoom' window ---*/
	ApCreateModule( ABM_zoom, NULL, NULL );

	/*--- Clear generation ---*/
	do_life_clear();

#if SIMULATION==1
	{
	static	UBYTE	mask[ 8 ]  = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
	UBYTE	*p, *ptr = bitmap_to_use();
	#define xy(x,y) \
		p = &ptr[ (y * FIELD_WIDTH8) + ( x / 8) ]; \
		*p |= mask[ x % 8 ]
	xy(398, 299); xy(399, 299); xy( 400, 299);
	xy(398, 300); xy(401, 300);
	xy(398, 301); xy(402, 301);
	damage_current_bitmap();
	show_life( 1 );
	do_zoom_redraw();
	ApModifyItemState( &file_menu, AB_ITEM_NORMAL, ABN_file_print, NULL );
	}
#endif

	/*--- Initialize statistics and show-it (if window is visible) ---*/
	memset( &gstats, 0, sizeof( STATS ) );
	show_stats();

	/*--- Life field: Show informations (Generation, Population) ---*/
	setString( ABW_status_gen, "%d", Generation );
	setString( ABW_status_pop, "%d", Population );
	setString( ABW_status_birth, "" );
	setString( ABW_status_death, "" );

	/*--- Options still not available ---*/
	ApModifyItemState( &lrules_menu, AB_ITEM_DIM, 
		ABN_lrules_load, ABN_lrules_save,
		NULL );

	return( Pt_CONTINUE );

	}	// life_init
