/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


static int initial_size_life( void )
	{
	PhRect_t	rect;
	PhDim_t		screen_size;
	static int	coefs[][ 3 ] =
		{
		{  640, 320, 240 },
		{  800, 400, 300 },
		{ 1024, 600, 450 },
		};
	int			k;
	BOOL		found;

	/*--- Get the screen size ---*/	
	PhWindowQueryVisible( Ph_QUERY_INPUT_GROUP, 0, 0, &rect );
	screen_size.w = (rect.lr.x - rect.ul.x + 1);
	screen_size.h = (rect.lr.y - rect.ul.y + 1);

	/*--- Adapt the size of the life window ---*/
	for ( found = FALSE, k = 0; k < (sizeof( coefs ) / sizeof( coefs[ 0 ] )); k++ )
		if ( screen_size.w <= coefs[ k ][ 0 ] )
			{
			found = TRUE;
			break;
			}
	if ( !found )
		return 0;

	/*--- Change the size of the window ---*/
	setDim( ABW_life, coefs[ k ][ 1 ], coefs[ k ][ 2 ] );

	return 0;
	
	}	// initial_size_life


int
life_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PhDim_t			prev_size, new_size;
	int				x_delta, y_delta;
	PhArea_t		*old_area, new_area;
	PgColor_t		colors[ 1 ];
	PhPoint_t		pos;
	PtArg_t			args[ 4 ];
	PhDim_t			size;
	PhArea_t		area;
	void			**xfield;

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Set the Position/Size of the life window if given in the .ini file ---*/
	getDim( ABW_life, &prev_size.w, &prev_size.h );
	if ( readandset_cfg_szpos( "Life window", ABW_life ) == -1 )
		initial_size_life();

	/*--- Show scrollbar if any ---*/
	getDim( ABW_life, &new_size.w, &new_size.h );
	x_delta = (new_size.w - prev_size.w);
	y_delta = (new_size.h - prev_size.h);
	if ( (x_delta != 0) || (y_delta != 0) )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
		PtGetResources( ABW_life_scrollarea, 1, args );
		memmove( &new_area, old_area, sizeof( PhArea_t ) );
		new_area.size.h += y_delta;
		new_area.size.w += x_delta;
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
		PtSetResources( ABW_life_scrollarea, 1, args );
		}	// if


	/*--- ----------------------------- ---*/
	/*--- Bitmap for use inside widgets ---*/
	/*--- ----------------------------- ---*/

	/*--- Create the 1st bitmap widget for the life field ---*/
	size.w = area.size.w = FIELD_WIDTH;
	size.h = area.size.h = FIELD_HEIGHT;
	pos.x = pos. y = area.pos.x = area.pos.y = 0;
	colors[ 0 ] = Pg_BLACK;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_DIM, &size, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_AREA, &area, 0 );
	PtSetArg( &args[ 3 ], Pt_ARG_BITMAP_COLORS, colors, 1 );
	PtSetResources( ABW_life_bmp1, 4, args );
	PtSetArg( &args[ 0 ], Pt_ARG_BITMAP_DATA, &xfield, 0 );
	PtGetResources( ABW_life_bmp1, 1, args );
	*xfield = field1;

	/*--- Create the 2nd bitmap widget for the life field ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_DIM, &size, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_AREA, &area, 0 );
	PtSetArg( &args[ 3 ], Pt_ARG_BITMAP_COLORS, colors, 1 );
	PtSetResources( ABW_life_bmp2, 4, args );
	PtSetArg( &args[ 0 ], Pt_ARG_BITMAP_DATA, &xfield, 0 );
	PtGetResources( ABW_life_bmp2, 1, args );
	*xfield = field2;

	/*--- Clear every thing ---*/
	show_life( 1 );

	return( Pt_CONTINUE );

	}	// life_setup
