/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


void show_which_rules( void )
	{
		int			k, z;
static	UBYTE		m[ 8 ] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };
		char		temp[ 80 ];
		PhRect_t	extent;
		int			x0, x, y;
		BOOL		use_prob;

	/*--- Are we using the standard rules ? ---*/	
	for ( use_prob = FALSE, k = 0; (k < 8) && (!use_prob); k++ )
		if ( (wrk_lrules.births_prob[ k ] != 0xFF) || (wrk_lrules.deaths_prob[ k ] != 0xFF) )
			use_prob = TRUE;
	if ( !use_prob &&
		 (wrk_lrules.births == STD_BIRTHS) && (wrk_lrules.deaths == STD_DEATHS) )
		{
		is_std_rules = TRUE;
		setString( ABW_status_lrules_title, "Rules: standard" );
		}
	else
		{
		is_std_rules = FALSE;
		setString( ABW_status_lrules_title, "Rules: non standard" );
		}

	/*--- Show births rules ---*/
	for ( strcpy( temp, "" ), z = 0, k = 0; k < 8; k++ )
		if ( wrk_lrules.births & m[ k ] )
			sprintf( &temp[ strlen( temp ) ], "%s%d",
				(z++ > 0) ? "," : "", k + 1 );
	setString( ABW_status_lrules_births, "%s", temp );
	getTextExtent( ABW_status_lrules_births, &extent );
	getPosWidget( ABW_status_lrules_births, &x0, NULL );
	getPosWidget( ABW_status_lrules_births_label, &x, &y );
	x = x0 + (extent.lr.x - extent.ul.x + 1) + 7;
	setPosWidget( ABW_status_lrules_births_label, x, y );
	
	/*--- Show deaths rules ---*/
	for ( strcpy( temp, "" ), z = 0, k = 0; k < 8; k++ )
		if ( wrk_lrules.deaths & m[ k ] )
			sprintf( &temp[ strlen( temp ) ], "%s%d",
				(z++ > 0) ? "," : "", k + 1 );
	setString( ABW_status_lrules_deaths, "%s", temp );
	getTextExtent( ABW_status_lrules_deaths, &extent );
	getPosWidget( ABW_status_lrules_deaths, &x0, NULL );
	getPosWidget( ABW_status_lrules_deaths_label, &x, &y );
	x = x0 + (extent.lr.x - extent.ul.x + 1) + 7;
	setPosWidget( ABW_status_lrules_deaths_label, x, y );

	}	// show_which_rules


void create_std_rule( LRULES *lrules )
	{
	int		k;

	lrules->births = STD_BIRTHS;			// 0000 0100
	lrules->deaths = STD_DEATHS;			// 0000 0110
	lrules->surviv = ~lrules->deaths;		// 1111 1001

	for ( k = 0; k < 8; k++ )
		lrules->births_prob[ k ] =
		lrules->deaths_prob[ k ] = 
		lrules->surviv_prob[ k ] = 0xFF;

	/*--- Are we using standard rules ? ---*/
	is_std_rules = TRUE;

	/*--- Refresh the rule window ? ---*/
	if ( ABW_lrules )
		update_lrules_window();

	/*--- Show which rules (status window title) ---*/
	show_which_rules();

	}	// create_std_rule


int
lrules_standard( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Create life standard rules ---*/
	create_std_rule( &wrk_lrules );

	return( Pt_CONTINUE );

	}	// lrules_standard


int
lrules_btn_cancel( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Ok, destroy the window ---*/
	PtDestroyWidget( ABW_lrules );

	return( Pt_CONTINUE );

	}	// lrules_btn_cancel


static do_lrules_useprob( void )
	{

	if ( isBtnSet( ABW_lrules_useprob ) )
		{
		PtRealizeWidget( ABW_lrules_births_prob );
		moveByWidget( ABW_lrules_births, 0, -13 );
		PtRealizeWidget( ABW_lrules_deaths_prob );
		moveByWidget( ABW_lrules_deaths, 0, -13 );
		PtRealizeWidget( ABW_lrules_surviv_prob );
		moveByWidget( ABW_lrules_surviv, 0, -13 );
		}
	else
		{
		PtUnrealizeWidget( ABW_lrules_births_prob );
		moveByWidget( ABW_lrules_births, 0, 13 );
		PtUnrealizeWidget( ABW_lrules_deaths_prob );
		moveByWidget( ABW_lrules_deaths, 0, 13 );
		PtUnrealizeWidget( ABW_lrules_surviv_prob );
		moveByWidget( ABW_lrules_surviv, 0, 13 );
		}

	}	// do_lrules_useprob


int
lrules_useprob( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Show/Hide probabilities ---*/
	do_lrules_useprob();

	return( Pt_CONTINUE );

	}	// lrules_useprob


int
lrules_btn_ok( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Births ---*/
#define UPDT( n, m ) \
	if ( isBtnSet( ABW_lrules_birth##n ) ) \
		wrk_lrules.births |= m; \
	else \
		wrk_lrules.births &= ~m;
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	/*--- Update deaths ---*/
#define UPDT( n, m ) \
	if ( isBtnSet( ABW_lrules_death##n ) ) \
		wrk_lrules.deaths |= m; \
	else \
		wrk_lrules.deaths &= ~m;
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	/*--- Update survivance ---*/
#define UPDT( n, m ) \
	if ( isBtnSet( ABW_lrules_surviv##n ) ) \
		wrk_lrules.surviv |= m; \
	else \
		wrk_lrules.surviv &= ~m;
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	/*--- If Ok, close this window ---*/
	PtDestroyWidget( ABW_lrules );

	/*--- Show which rules (status window title) ---*/
	show_which_rules();

	return( Pt_CONTINUE );

	}	// lrules_btn_ok


static void update_lrules_window( void )
	{
	int		k;
	BOOL	use_prob;

	/*--- Using probabilities ? ---*/
	for ( use_prob = FALSE, k = 0; (k < 8) && (!use_prob); k++ )
		if ( (wrk_lrules.births_prob[ k ] != 0xFF) || (wrk_lrules.deaths_prob[ k ] != 0xFF) )
			use_prob = TRUE;

	/*--- Update prob. fields ---*/
	setBtn( ABW_lrules_useprob, use_prob );

	/*--- Update births ---*/
#define UPDT( n, m ) setBtn( ABW_lrules_birth##n, wrk_lrules.births & m )
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	/*--- Update deaths ---*/
#define UPDT( n, m ) setBtn( ABW_lrules_death##n, wrk_lrules.deaths & m )
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	/*--- Update survivance ---*/
#define UPDT( n, m ) setBtn( ABW_lrules_surviv##n, wrk_lrules.surviv & m )
	UPDT( 1, 0x01 );
	UPDT( 2, 0x02 );
	UPDT( 3, 0x04 );
	UPDT( 4, 0x08 );
	UPDT( 5, 0x10 );
	UPDT( 6, 0x20 );
	UPDT( 7, 0x40 );
	UPDT( 8, 0x80 );
#undef UPDT

	}	// update_lrules_window


int
lrules_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Set the Position/Size of the window if given in the .ini file ---*/
	readandset_cfg_szpos( "Define rules window", ABW_lrules );

	/*--- Update the window ---*/
	update_lrules_window();

	return( Pt_CONTINUE );

	}	// lrules_setup


int
lrules_realized( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Show/Hide probabilities ---*/
	do_lrules_useprob();

	return( Pt_CONTINUE );

	}	// lrules_realized


int
lrules_destroyed( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Save position and dimension of this window into the config (life.ini) file ---*/
	save_posdim_in_config_file( "Define rules window", ABW_lrules );

	/*--- Dialog not mounted ---*/
	ABW_lrules = NULL;

	return( Pt_CONTINUE );

	}	// lrules_destroyed


int
lrules_define( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Open the window, only of already not opened ---*/
	if ( ABW_lrules == NULL )
		ApCreateModule( ABM_lrules, NULL, NULL );
	else
		window_focus( ABW_lrules );

	return( Pt_CONTINUE );

	}	// lrules_define

int
status_lrules( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Open the window, only of already not opened ---*/
	if ( ABW_lrules == NULL )
		ApCreateModule( ABM_lrules, NULL, NULL );
	else
		window_focus( ABW_lrules );

	return( Pt_CONTINUE );

	}	// status_lrules

int
lrules_deaths( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Update the survivance rules ---*/
#define UPDT( n ) \
	setBtn( ABW_lrules_surviv##n, !isBtnSet( ABW_lrules_death##n ) )
	UPDT( 1 );
	UPDT( 2 );
	UPDT( 3 );
	UPDT( 4 );
	UPDT( 5 );
	UPDT( 6 );
	UPDT( 7 );
	UPDT( 8 );
#undef UPDT

	return( Pt_CONTINUE );

	}	// lrules_deaths
