/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


/*--- ------------------------------ ---*/
/*--- Local variables to this module ---*/
/*--- ------------------------------ ---*/

static int	prev_opt_x1, prev_opt_x2, prev_opt_y1, prev_opt_y2;


static void test_gen_stable( const UBYTE *curr_gen )
	{
	int		n;

	/*--- If neither birth nor death, population is stable ---*/
	if ( (births == 0) && (deaths == 0) )
		{

		/*--- Show into life field window ---*/
		setString( ABW_status_stable_period, "1" );
		PtRealizeWidget( ABW_status_stable_label );
		PtRealizeWidget( ABW_status_stable_period );

			/*--- Population stable ---*/
		is_stability = TRUE;
		stability_period = 1;
		return;
		}	// if

	/*--- Compare with some previous generations ---*/
	for ( n = gen_kept_nb - 1; n >= 0; n-- )
		{
		if ( memcmp( curr_gen, gen_kept[ n ], FIELD_SIZE ) == 0 )
			{

			/*--- Show into life field window ---*/
			setString( ABW_status_stable_period, "%d", gen_kept_nb - n );
			PtRealizeWidget( ABW_status_stable_label );
			PtRealizeWidget( ABW_status_stable_period );

			/*--- Population stable ---*/
			is_stability = TRUE;
			stability_period = gen_kept_nb - n;
			return;
			}
		}
	
	}	// test_gen_stable


static void test_pop_stable( void )
	{
	int		n;
	int		val;

	if ( gen_kept_nb < max_nb_gen_kept )
		return;

	val = gen_kept_popnb[ 0 ];
	for ( n = 1; n < gen_kept_nb; n++ )
		if ( gen_kept_popnb[ n ] != val )
			return;

	/*--- Show into life field window ---*/
	setString( ABW_status_stable_period, "%d", gen_kept_nb );
	PtRealizeWidget( ABW_status_stable_label );
	PtRealizeWidget( ABW_status_stable_period );

	is_stability = TRUE;
	stability_period = gen_kept_nb;
	
	}	// test_gen_stable


static void extend_opti( void )
	{

	if ( --opt_y1 < 1 )
		opt_y1 = 1;
	if ( ++opt_y2 > (FIELD_HEIGHT - 2) )
		opt_y2 = FIELD_HEIGHT - 2;

	if ( --opt_x1 < 1 )
		opt_x1 = 1;
	if ( ++opt_x2 > (FIELD_WIDTH8 - 2) )
		opt_x2 = FIELD_WIDTH8 - 2;

	}	// extend_opti


static void fast_draw( UBYTE *p0 )
	{
static	PhPoint_t	size = { FIELD_WIDTH, FIELD_HEIGHT };	// Should be PhSize_t (cf. doc)
static	PgColor_t	palette[ 2 ] = { Pg_GREY, Pg_BLACK };
		PhPoint_t	pos;
		PtArg_t		args[ 2 ];
		USHORT		*currpos_x, *currpos_y;
		PhRect_t	rect;

	/*--- Get current scrollbar position into the life virtual view ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_SCROLL_AREA_POS_X, &currpos_x, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_SCROLL_AREA_POS_Y, &currpos_y, 0 );
	PtGetResources( ABW_life_scrollarea, 2, args );
	pos.x = -*currpos_x;
	pos.y = -*currpos_y;

	/*--- Find the biggest opti rectangle btw the last and the current ---*/
	rect.ul.x = min( prev_opt_x1, opt_x1 ) * 8 + pos.x;
	rect.lr.x = max( prev_opt_x2, opt_x2 ) * 8 + pos.x;
	rect.ul.y = min( prev_opt_y1, opt_y1 ) + pos.y;
	rect.lr.y = max( prev_opt_y2, opt_y2 ) + pos.y;

	bitmap_to_bytenibble( p0, fast_field );
	PgSetRegion( PtWidgetRid( ABW_life_scrollarea ) );
	PgSetUserClip( &rect );
	PgSetPalette( palette, 0, 0, 2, Pg_PALSET_SOFT, 0 );
	PgDrawImagemx( fast_field, Pg_IMAGE_PALETTE_NIBBLE, &pos, &size, FIELD_WIDTH/2, 0 );
	PgFlush();

	}	// fast_draw


void do_compute_next_gen( const BOOL is_fast )
	{
	UBYTE		*p0;
	clock_t		t1, t2, t3, t4, t5;
	BOOL		is_updatelife;

	/*--- Save generation #0 ---*/
	if ( Generation == 0 )
		{
		new_graph_data( Generation, Population, 0, 0, 0 );
		GENMOVE( gen0_kept, bitmap_to_use() );
		}	// if

	/*--- Get bitmaps addresses ---*/
	t1 = clock();

	/*--- Draw life ? ---*/
	if ( (ABW_seek != NULL) && (!isBtnSet( ABW_seek_is_updatelife )) )
		is_updatelife = FALSE;
	else
		is_updatelife = TRUE;

	/*--- Compute the next generation ---*/
	prev_opt_x1 = opt_x1;
	prev_opt_x2 = opt_x2;
	prev_opt_y1 = opt_y1;
	prev_opt_y2 = opt_y2;
	if ( (++Generation % 2) == 1 )
		{
		t2 = clock();
		compute_gen( p0 = field1, field2 );
		t3 = clock();
		extend_opti();
		if ( is_updatelife )
			if ( is_fast )
				fast_draw( p0 );
			else
				show_life( 2 );
		}
	else
		{
		t2 = clock();
		compute_gen( p0 = field2, field1 );
		t3 = clock();
		extend_opti();
		if ( is_updatelife )
			if ( is_fast )
				fast_draw( p0 );
			else
				show_life( 1 );
		}
	t4 = clock();

	/*--- Life field: Draw informations (Generation, Population) ---*/
	setString( ABW_status_gen, "%d", Generation );
	setString( ABW_status_pop, "%d", Population );
	setString( ABW_status_birth, "%d", births );
	setString( ABW_status_death, "%d", deaths );

	/*--- Redraw the zoom pane ---*/
	do_zoom_redraw();

	/*--- Update the generations kept to see if ... ---*/
	if ( gen_kept_nb < max_nb_gen_kept )
		{
		gen_kept_popnb[ gen_kept_nb ] = Population;
		GENMOVE( gen_kept[ gen_kept_nb++ ], p0 );
		}
	else
		{
		memmove( &gen_kept_popnb[ 0 ],
				 &gen_kept_popnb[ 1 ],
				 sizeof( int ) * (gen_kept_nb - 1) );
		gen_kept_popnb[ gen_kept_nb - 1 ] = Population;
#if USE_ASM == 2
		gencopy( &gen_kept_base[ 0 ], 
				 &gen_kept_base[ FIELD_SIZE ], 
				 (FIELD_SIZE * (gen_kept_nb - 1)) >> 2 );
#else
		memmove( &gen_kept_base[ 0 ], 
				 &gen_kept_base[ FIELD_SIZE ], 
				 FIELD_SIZE * (gen_kept_nb - 1) );
#endif
		GENMOVE( gen_kept[ gen_kept_nb - 1 ], p0 );
		}

	/*--- Test if this generation is stable ---*/
	test_gen_stable( bitmap_to_use() );

	/*--- Test if population number if stable ---*/
	if ( is_snp_abort )
		test_pop_stable();

	/*--- Enable previous generation ---*/
	ApModifyItemState( &life_menu, AB_ITEM_NORMAL, 
		ABN_life_prev, ABN_life_restoregen0,
		NULL );

	/*--- Update and show statistics (if dialogis visible) ---*/
	t5 = clock();
	gstats.total_pop += Population;
	gstats.total_births += births;
	gstats.total_deaths += deaths;
	gstats.total_surviv += surviv;
	gstats.total_ctime += ((t3 - t2) / (CLOCKS_PER_SEC / 1000));
	gstats.total_dtime += ((t4 - t3) / (CLOCKS_PER_SEC / 1000));
	gstats.total_ttime += ((t5 - t1) / (CLOCKS_PER_SEC / 1000));
	show_stats();

	/*--- Add a new graph data sample ---*/
	new_graph_data( Generation, Population, births, deaths, surviv );

	}	// do_compute_next_gen


int
next_gen( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Set to TRUE while computing next generations (Shift-F2) ---*/
	if ( is_comp_generations )
		return( Pt_CONTINUE );

	/*--- Do nothing if population empty ! ---*/
	if ( Population == 0 )
		return( Pt_CONTINUE );

	/*--- Compute now the next generation ---*/
	busyWindow( ABW_zoom, TRUE );
	busyWindow( ABW_life, TRUE );
	do_compute_next_gen( FALSE );
	busyWindow( ABW_life, FALSE );
	busyWindow( ABW_zoom, FALSE );

	/*--- Enable/Disable option 'Save' of the menu ---*/
	ApModifyItemState( &file_menu, 
		(Population > 0) ? AB_ITEM_NORMAL : AB_ITEM_DIM, 
		ABN_file_save, ABN_file_save_text, ABN_file_print,
		NULL );

	/*--- Enable the button 'Reload starting generation', only of seek window is open ---*/
	if ( ABW_seek )
		{
		ghostBtn( ABW_seek_btn_reloadstartgen, FALSE );
		ghostBtn( ABW_seek_btn_reloadbestgen,  FALSE );
		}

#if LOGTOTMP==1
	slog( "compute a new generation" );
#endif

	return( Pt_CONTINUE );

	}	// next_gen
