/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


int
next_generations( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	time_t		start_nextgens;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Set to TRUE while computing next generations (Shift-F2) ---*/
	if ( is_comp_generations )
		return( Pt_CONTINUE );

	/*--- Do nothing if population empty ! ---*/
	if ( Population == 0 )
		return( Pt_CONTINUE );

	/*--- Iconify the zoom window ---*/
	iconify_window( ABW_zoom, TRUE );

	/*--- Disable all access to the zoom window ---*/
	busyWindow( ABW_zoom, TRUE );

	/*--- Show 'Stop' button on the life field window ---*/
	PtRealizeWidget( ABW_status_btn_stop );

	/*--- Set to TRUE while computing next generations (Shift-F2) ---*/
	is_comp_generations = TRUE;

	/*--- Life field: Clear stable infos ---*/
	PtUnrealizeWidget( ABW_status_stable_label );
	PtUnrealizeWidget( ABW_status_stable_period );

	/*--- Generations kept to see if stability ---*/
	is_stability = FALSE;

	/*--- If seek window is open, then block all input ---*/
	if ( ABW_seek )
		busyWindow( ABW_seek, TRUE );

	/*--- Field of life has focus ---*/
	window_focus( ABW_life );

	/*--- Hide the graphics window if opened ---*/
	if ( ABW_graph != NULL )
		PtUnrealizeWidget( ABW_graph );

	/*--- To compute time for doing complete next generations ---*/
	start_nextgens = time( NULL );

	/*--- Compute next generations until 
	      - the button 'Stop' is pressed
		  - the generation is stable or dead. ---*/
	for ( end_comp_generations = FALSE; !end_comp_generations; )
		{

		/*--- Compute the next generation ---*/
		do_compute_next_gen( TRUE );

		/*--- End if no population or stability ---*/
		if ( (Population == 0) || (is_stability) )
			break;

		/*--- Let Photon handle pending events ---*/
		PtBkgdHandlerProcess();
		
		}	// for (;;)
	is_comp_generations = FALSE;

	/*--- Stop computing generations ---*/
	PtUnrealizeWidget( ABW_status_btn_stop );

	/*--- To compute time for doing complete next generations ---*/
	nextgens_timeelps = difftime( time( NULL ), start_nextgens );
	if ( ABW_stats )
		setString( ABW_stats_lastnextgens, "%d", nextgens_timeelps );

	/*--- Des-hide the graphics window if opened ---*/
	if ( ABW_graph != NULL )
		PtRealizeWidget( ABW_graph );

	/*--- Restore access to the zoom window ---*/
	busyWindow( ABW_zoom, FALSE );

	/*--- If seek window is open, then unblock all input ---*/
	/*--- Enable the button 'Reload starting generation', only of seek window is open ---*/
	if ( ABW_seek )
		{
		busyWindow( ABW_seek, FALSE );
		ghostBtn( ABW_seek_btn_reloadstartgen, FALSE );
		ghostBtn( ABW_seek_btn_reloadbestgen,  FALSE );
		}

	/*--- Enable/Disable option 'Save' of the menu ---*/
	ApModifyItemState( &file_menu, 
		(Population > 0) ? AB_ITEM_NORMAL : AB_ITEM_DIM, 
		ABN_file_save, ABN_file_save_text, ABN_file_print,
		NULL );

	/*--- Des-Iconify the zoom window ---*/
	iconify_window( ABW_zoom, FALSE );
	window_focus( ABW_zoom );

#if LOGTOTMP==1
	slog( "compute new generations" );
#endif

	return( Pt_CONTINUE );

	}	// next_generations
