/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"

#define HMARGIN	0
#define VMARGIN 5

#define CELLP_HEIGHT 10
#define CELLP_WIDTH  10

static	PpPrintContext_t	*printCtxt;

static	PtWidget_t			*pwin;
static	PtWidget_t			*ppane;
static	PtWidget_t			*ppage;


int
file_print( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Dont' do while computing next generations (Shift-F2) ---*/
	if ( is_comp_generations )
		return( Pt_CONTINUE );

	/*--- Open the 'Print' window ---*/
	ApCreateModule( ABM_print, NULL, NULL );

	return( Pt_CONTINUE );

	}	// file_print


int
print_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- "Print only this generation ... " ---*/
	setString( ABW_print_curr_gen, "%d", Generation );

	/*--- "Print only generations between ... and ... " ---*/
	setString( ABW_print_start_gen, "1" );
	setString( ABW_print_end_gen,   "%d", Generation );

	/*--- Enable the button 'Print' only if non empty generation ---*/
	ghostBtn( ABW_print_btn_print, (Population == 0) );

	return( Pt_CONTINUE );

	}	// print_setup


static void find_real_size( int *sx1, int *sx2, int *sy1, int *sy2 )
	{
		int		row, col, rcol;
		int		w, k;
static	UBYTE	masks[ 8 ] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
const 	UBYTE	*p, *q;

	*sx1 = *sy1 = INT_MAX;
	*sx2 = *sy2 = INT_MIN;
	
	p = bitmap_to_use();
	for ( row = 0; row < FIELD_HEIGHT; row++ )
		for ( col = 0; col < FIELD_WIDTH8; col++ )
			for ( rcol = col<<3, w = *p++, q = masks, k = 8; k; k--, rcol++ )
				if ( w & *q++ )
					{
					if ( rcol < *sx1 )	*sx1 = rcol;
					if ( rcol > *sx2 )	*sx2 = rcol;
					if ( row < *sy1 )	*sy1 = row;
					if ( row > *sy2 )	*sy2 = row;
					}

	}	// find_real_size


/* static */ void do_print( void )
	{
		int			sx1, sx2, sy1, sy2;
		char		label[ 80 ];
		PtArg_t		args[ 10 ];
		PtWidget_t	*pgrp_pop;
		PtWidget_t	*plabel;
		PtWidget_t	*pgrp, *pgrp_tmp;
		PhArea_t	area;
		PtWidget_t	*pcell;
		int			x, y;
static	UBYTE		masks[ 8 ] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
		UBYTE		*p;

	/*--- Find the dimension of generation to print ---*/
	find_real_size( &sx1, &sx2, &sy1, &sy2 );
printf( " 1) %d,%d  %d,%d\n", sx1, sx2, sy1, sy2 );

	/*--- Create a group to draw: population + label ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_WHITE, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_GROUP_HORZ_ALIGN, Pt_GROUP_HORZ_CENTER, 0 );
	PtSetArg( &args[ 3 ], Pt_ARG_GROUP_VERT_ALIGN, Pt_GROUP_VERT_BOTTOM, 0 );
	PtSetArg( &args[ 4 ], Pt_ARG_RESIZE_FLAGS, Pt_RESIZE_XY_ALWAYS, 0 );
	pgrp = PtCreateWidget( PtGroup, ppage, 4, args );
	PtRealizeWidget( pgrp );

	/*--- Create a group to draw: population + label ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_YELLOW, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_ANCHOR_FLAGS, Pt_TOP_ANCHORED_BOTTOM, 0 );
	pgrp_tmp = PtCreateWidget( PtGroup, pgrp, 2, args );
	PtRealizeWidget( pgrp_tmp );

	/*--- Create the drawing of this generation: population + label ---*/
	sprintf( label, "Gen %d - Pop=%d", Generation, Population );
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_TEXT_STRING, label, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_TEXT_FONT, "helv24b", 0 );
	plabel = PtCreateWidget( PtLabel, pgrp_tmp, 3, args );
	PtRealizeWidget( plabel );

	/*--- Create a group to store all the population ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_ASIS, 0 );
	pgrp_pop = PtCreateWidget( PtGroup, pgrp, 2, args );
	PtRealizeWidget( pgrp_pop );

	/*--- Create the generation ---*/
	area.size.w = CELLP_WIDTH;
	area.size.h = CELLP_HEIGHT;
	p = bitmap_to_use();
	for ( x = sx1; x <= sx2; x++ )
		for ( y = sy1; y <= sy2; y++ )
			if ( p[ y*FIELD_WIDTH8 + (x>>3) ] & masks[ x%8 ] )
				{
				area.pos.x = (x-sx1) * CELLP_WIDTH;
				area.pos.y = (y-sy1) * CELLP_HEIGHT;
				PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
				PtSetArg( &args[ 1 ], Pt_ARG_COLOR, Pg_BLUE, 0 );
				PtSetArg( &args[ 2 ], Pt_ARG_FILL_COLOR, Pg_BLUE, 0 );
				pcell = PtCreateWidget( PtEllipse, pgrp_pop, 3, args );
				PtRealizeWidget( pcell );
				}
	PtExtentWidget( pgrp_pop );

	PtExtentWidget( pgrp );

	}	// do_print


int
print_btn_print( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	void			*pcdata;
	PhArea_t		area;
	PtArg_t			args[ 10 ];
	PhDim_t			paper_size, size;
	PhRect_t		*prect;
	int				h, w;
	int				what_print;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Create the print context ---*/
	if ( (printCtxt = PpPrintCreatePC()) == NULL )
		return( Pt_CONTINUE );

	/*--- Dialog to choose the printer ---*/	
	if ( (what_print=PtPrintSelection( ABW_print,	// Parent
			NULL,									// Position (centered)
			"Print Life",							// Title
			printCtxt,								// Print context
			Pt_PRINTSEL_ALL_PANES | Pt_PRINTSEL_PROP_APP |
			Pt_PRINTSEL_NO_COPIES | Pt_PRINTSEL_NO_SELECT_RANGE |
			Pt_PRINTSEL_NO_COLLATE
			)) == Pt_PRINTSEL_CANCEL )
		return( Pt_CONTINUE );

	/*--- ---*/
	PpPrintGetPC( printCtxt, Pp_PC_PAPER_SIZE, &pcdata );
	paper_size.w = ((PhDim_t*)pcdata)->w;
	paper_size.h = ((PhDim_t*)pcdata)->h;
	PpPrintGetPC( printCtxt, Pp_PC_NONPRINT_MARGINS, &prect );
printf( "paper_size: w=%d h=%d\n", paper_size.w, paper_size.h );

	size.w = ((paper_size.w - (prect->ul.x + prect->lr.x)) * 72) / 250; 
	size.h = ((paper_size.h - (prect->ul.y + prect->lr.y)) * 72) / 250; 
	PpPrintSetPC( printCtxt, INITIAL_PC, 0, Pp_PC_SOURCE_SIZE, &size ); 

	/*--- Create the window to print ---*/
	area.pos.x = 300;
	area.pos.y = 815;
	w = area.size.w = paper_size.w / 5;
	h = area.size.h = paper_size.h / 5;
	w = area.size.w = size.w;
	h = area.size.h = size.h;
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_FILL_COLOR, Pg_WHITE, 0 );
	pwin = PtCreateWidget( PtWindow, ABW_print, 2, args );
	PtRealizeWidget( pwin );

	/*--- Create a pane inside the page ---*/
	area.pos.x = (w * HMARGIN) / 100;
	area.pos.y = (h * VMARGIN) / 100;
	area.size.w = w - 2*area.pos.x;
	area.size.h = h - 2*area.pos.y;
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_FILL_COLOR, Pg_MGREY, 0 );
	ppane = PtCreateWidget( PtPane, pwin, 2, args );
	PtRealizeWidget( ppane );

	/*--- Create a group to store all the populations (one page) ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, Pg_RED, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_HORIZONTAL, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_BORDER_WIDTH, 2, 0 );
	PtSetArg( &args[ 3 ], Pt_ARG_GROUP_SPACING_X, 10, 0 );
	PtSetArg( &args[ 4 ], Pt_ARG_GROUP_SPACING_Y, 4, 0 );
	PtSetArg( &args[ 5 ], Pt_ARG_GROUP_FLAGS, Pt_TRUE, Pt_GROUP_EQUAL_SIZE_VERTICAL);
	ppage = PtCreateWidget( PtGroup, ppane, 6, args );
	PtRealizeWidget( ppage );

	/*--- Compute next generations until 
	      - the button 'Stop' is pressed
		  - the generation is stable or dead. ---*/
	PpPrintOpen( printCtxt );
	for ( end_comp_generations = FALSE; !end_comp_generations; )
		{

		/*--- Print the current generation ---*/
		do_print();
		PtExtentWidget( ppage );

if ( Generation==12 ) {
		PpPrintStart( printCtxt );
		PpPrintWidget( printCtxt, pwin, NULL, NULL, Pt_PP_RESIZE_PC );
		PtFlush();
		PpPrintStop( printCtxt );		
return( Pt_CONTINUE );
{
FILE *fp;
PtFlush();
printf( "enter 'Z': " );
fflush( stdout );
fp = fopen( "/dev/ttyp0", "r" );
while (getc(fp) != 'Z' )
	PtBkgdHandlerProcess();
fclose( fp );
}
}

		/*--- Compute the next generation ---*/
		do_compute_next_gen( TRUE );

		/*--- End if no population or stability ---*/
		if ( (Population == 0) || (is_stability) )
			break;

		/*--- Let Photon handle pending events ---*/
		PtBkgdHandlerProcess();

		}	// for (;;)
	is_comp_generations = FALSE;

	/*--- Release the print context ---*/
	PpPrintClose( printCtxt );		
	PpPrintReleasePC( printCtxt );

	return( Pt_CONTINUE );

	}	// print_btn_print
