/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


#define ISLIVE( x, y ) \
	(p[ ((ys + (y)) * FIELD_WIDTH8) + ((xs + (x)) >> 3) ] & mSet[ (xs + (x)) % 8 ] )

#define SET( x, y ) \
	p[ ((ys + (y))  * FIELD_WIDTH8) + ((xs + (x)) >> 3) ] |= mSet[  (xs + (x)) % 8 ]; \
	Population++

#define BTW( a, b ) \
	(rand() % ((b) - (a) + 1)) + (a)


void compute_random_generation( int tsize_width1, int tsize_width2, 
							    int tsize_height1, int tsize_height2,
							    int pop_min, int pop_max )
	{
		int		xs, ys;
		int		xp, yp;
		int		xn, yn;
		int		asked_cells;
static	UBYTE	mSet[ 8 ] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
		UBYTE	*p;
		BYTE	symetry;
		int		x, y;
		int		width, height;
		int		offset_width, offset_height;
		int		cnt;
		int		tsize_width, tsize_height;

	/*--- Order... ---*/
#define ORDER( a, b ) \
	if ( a > b ) \
		{ \
		int temp; \
		temp = a; \
		a = b; \
		b = temp; \
		}
	ORDER( pop_min, pop_max )
	ORDER( tsize_width1, tsize_width2 )
	ORDER( tsize_height1, tsize_height2 )
#undef ORDER

	/*--- Generation of a size ---*/
	tsize_width  = BTW( tsize_width1,  tsize_width2 );
	tsize_height = BTW( tsize_height1, tsize_height2 );

	/*--- Which symetry ? ---*/
	if ( isBtnSet( ABW_seek_symetry_hor ) )
		{ 
		symetry = 1;
		width  = (tsize_width/2) + (tsize_width % 2);
		height = tsize_height; 
		}
	else if ( isBtnSet( ABW_seek_symetry_vert ) )
		{ 
		symetry = 2;
		width  = tsize_width;
		height = (tsize_height/2) + (tsize_height % 2);
		}
	else if ( isBtnSet( ABW_seek_symetry_both ) )
		{ 
		symetry = 3;
		width  = (tsize_width/2)  + (tsize_width % 2);
		height = (tsize_height/2) + (tsize_height % 2);
		}
	else
		{ 
		symetry = 0;
		width  = tsize_width;
		height = tsize_height; 
		}

	/*--- Select the position inside the life field ---*/
	xs = (((FIELD_WIDTH  - tsize_width)  >> 1) >> 3) << 3;
	ys = (((FIELD_HEIGHT - tsize_height) >> 1) >> 3) << 3;

	/*--- First select a point inside the asked rectangle ---*/
	xp = rand() % width;
	yp = rand() % height;

	if ( pop_min > (width * height) )
		pop_min = pop_max = (width * height);
	asked_cells = BTW( pop_min, pop_max );

	/*--- Found the right number of cells ---*/
	for ( cnt = 0, p = bitmap_to_use(); ; )
		{

		/*--- Security... ---*/
		if ( ++cnt == 10000 )
			break;

		/*--- Take this point ---*/
		if ( !ISLIVE( xp, yp ) )
			{
			SET( xp, yp );
			if ( Population == asked_cells )
				break;
			}

		/*--- Compute next point ---*/
		for (;;)
			{
			xn = xp + (rand() % 3) - 1;
			yn = yp + (rand() % 3) - 1;
			if ( (xn != xp) || ( yn != yp) )
				if ( (xn >= 0) && (xn < width) && (yn >= 0) && (yn < height) )
					break;
			}
		xp = xn;
		yp = yn;
			
		}	// for

	/*--- Horizontal Symetry ? ---*/
	if ( (symetry >= 1) && (symetry <= 3) )
		{
		offset_width  = (tsize_width % 2)  ? 0 : 1;
		offset_height = (tsize_height % 2) ? 0 : 1;
		for ( x = 0; x < width; x++ )
			{
			for ( y = 0; y < height; y++ )
				{
				if ( ISLIVE( x, y ) )
					{
					switch ( symetry )
						{
						case 1 : 
							SET( tsize_width - offset_width - x, y ); 
							break;
						case 2 : 
							SET( x, tsize_height - offset_height - y );
							break;
						case 3 : 
							SET( tsize_width - offset_width - x, y ); 
							SET( x, tsize_height - offset_height - y );
							SET( tsize_width - offset_width - x, tsize_height - offset_height - y );
							break;
						}
					}	// if
				}	// for (y)
			}	// for (x)
		}	// if

	/*--- Redraw all the zoom area ---*/
	do_zoom_redraw();

	/*--- Generation kept in case of automatic/manual seeking ---*/
	GENMOVE( gen_start_kept_seek, bitmap_to_use() );

	/*--- Life field: Show informations (Generation, Population) ---*/
	setString( ABW_status_gen, "%d", Generation );
	setString( ABW_status_pop, "%d", Population );

	}	// compute_random_generation

#undef IS_FREE
#undef SET


int
seek_btn_genrnd( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- First clear generation ---*/
	do_life_clear();

	/*--- Compute a random generation and save-it ---*/
	compute_random_generation(
		getInt( ABW_seek_size_width1),   getInt( ABW_seek_size_width2), 
		getInt( ABW_seek_size_height1 ), getInt( ABW_seek_size_height2 ),
		getInt( ABW_seek_pop_min ),	     getInt( ABW_seek_pop_max ) );

	return( Pt_CONTINUE );

	}	// seek_btn_genrnd
