/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"


int
settings_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	unsigned int	kmem_avl;

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Grid ---*/
	setBtn( ABW_settings_grid_yes, is_grid );
	setBtn( ABW_settings_grid_no, !is_grid );

	/*--- SNP ---*/
	setBtn( ABW_settings_snp_yes, is_snp_abort );
	setBtn( ABW_settings_snp_no, !is_snp_abort );

	/*--- Generations kept ---*/
	setInt( ABW_settings_nbgen_kept, max_nb_gen_kept );
	setString( ABW_settings_mem_needed, "%d K", 
		(FIELD_SIZE * max_nb_gen_kept) / 1024 );
	_nheapgrow();
	kmem_avl = (_freect( FIELD_SIZE ) * FIELD_SIZE) / 1024;
	if ( (kmem_avl / 1024) >= 4 )
		setString( ABW_settings_mem_avail, "%d M", kmem_avl / 1024 );
	else
		setString( ABW_settings_mem_avail, "%d K", kmem_avl );

	/*--- Set the Position/Size of the window if given in the .ini file ---*/
	readandset_cfg_szpos( "Settings window", ABW_settings );

	return( Pt_CONTINUE );

	}	// settings_setup


int
settings_destroyed( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Save position and dimension of this window into the config (life.ini) file ---*/
	save_posdim_in_config_file( "Settings window", ABW_settings );

	/*--- This dialog is no more mounted ---*/
	ABW_settings = NULL;

	return( Pt_CONTINUE );

	}	// settings_destroyed


int
settings_btn_ok( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int		kept;
	int		n;
#ifdef NEVER
	int		avail;
#endif

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Grid ---*/
	is_grid = isBtnSet( ABW_settings_grid_yes );

	/*--- Generations kept ---*/
	is_snp_abort = isBtnSet( ABW_settings_snp_yes );

	/*--- Read numeric value ---*/
	kept = getInt( ABW_settings_nbgen_kept );
#ifdef NEVER
	avail = _memmax() * 1024;
	if ( ((kept * FIELD_SIZE * 110) / 100) >= avail )
		kept = ((avail * 100) / 110) / FIELD_SIZE;
#endif
	if ( kept != max_nb_gen_kept )
		{

		free( gen_kept );
		free( gen_kept_base );
		free( gen_kept_popnb );
		
		max_nb_gen_kept = kept;
		gen_kept_popnb = app_calloc( max_nb_gen_kept, sizeof( int ) );
		gen_kept = app_calloc( max_nb_gen_kept, sizeof( UBYTE * ) );
		gen_kept_base = app_calloc( max_nb_gen_kept, FIELD_SIZE );
		for ( n = 0; n < max_nb_gen_kept; n++ )
			gen_kept[ n ] = &gen_kept_base[ FIELD_SIZE * n ];
		gen_kept_nb = 0;

		}	// if

	/*--- Redraw the zoom area ---*/
	do_zoom_redraw();

	/*--- Close the window ---*/
	PtDestroyWidget( ABW_settings );

	return( Pt_CONTINUE );

	}	// settings_btn_ok


int
settings_nbgen_kept( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int		kept, need;
#ifdef NEVER
	int		avail;
#endif

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Read numeric value ---*/
	kept = getInt( widget );
	setString( ABW_settings_mem_needed, "%d K", 
		need = (FIELD_SIZE * kept) / 1024 );

#ifdef NEVER
	avail = _memmax() * 1024;
	if ( ((need * 110) / 100) >= avail )
		setColor( ABW_settings_mem_needed, Pg_RED );
	else
		setColor( ABW_settings_mem_needed, Pg_BLACK );
#endif

	return( Pt_CONTINUE );

	}	// settings_nbgen_kept
