/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"

void do_zoom_resize( const BOOL first, const int x_delta, const int y_delta )

	{
	PhArea_t	*old_area, new_area;
	PtArg_t		args[ 1 ];
	PhDim_t		*pane_dim;

	/*--- Move the zoom area pane ---*/
	if ( !first )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
		PtGetResources( ABW_zoom_area, 1, args );
		memmove( &new_area, old_area, sizeof( PhArea_t ) );
		new_area.size.h += y_delta;
		new_area.size.w += x_delta;
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
		PtSetResources( ABW_zoom_area, 1, args );
		}

	/*--- Compute how many cells will be shown ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &pane_dim, 0 );
	PtGetResources( ABW_zoom_area, 1, args );
	zoomed_cells_width_nb  = pane_dim->w / (CELLZ_WIDTH  + CELLZ_XGAP);
	zoomed_cells_height_nb = pane_dim->h / (CELLZ_HEIGHT + CELLZ_YGAP);

	/*--- Move the x-scrollbar ---*/
	if ( !first )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
		PtGetResources( ABW_zoom_area_xscrollbar, 1, args );
		memmove( &new_area, old_area, sizeof( PhArea_t ) );
		new_area.pos.y += y_delta;
		new_area.size.w += x_delta;
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
		PtSetResources( ABW_zoom_area_xscrollbar, 1, args );
		}

	/*--- Move the y-scrollbar ---*/
	if ( !first )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
		PtGetResources( ABW_zoom_area_yscrollbar, 1, args );
		memmove( &new_area, old_area, sizeof( PhArea_t ) );
		new_area.pos.x += x_delta;
		new_area.size.h += y_delta;
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
		PtSetResources( ABW_zoom_area_yscrollbar, 1, args );
		}

	/*--- Move the informations pane ---*/
	if ( !first )
		{
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &old_area, 0 );
		PtGetResources( ABW_zoom_pane_infos, 1, args );
		memmove( &new_area, old_area, sizeof( PhArea_t ) );
		new_area.pos.y += y_delta;
		new_area.size.w += x_delta;
		PtSetArg( &args[ 0 ], Pt_ARG_AREA, &new_area, 0 );
		PtSetResources( ABW_zoom_pane_infos, 1, args );
		}

	}	// do_zoom_resize


int
zoom_resize( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
		PtContainerCallback_t	*infos = (PtContainerCallback_t *)cbinfo->cbdata;
		int						x_delta, y_delta;
static	BOOL					first = TRUE;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Compute the difference ---*/
	x_delta = (infos->new_size.lr.x - infos->old_size.lr.x);
	y_delta = (infos->new_size.lr.y - infos->old_size.lr.y);

	/*--- Move the zoom area pane, and also the scrollbars ---*/
	do_zoom_resize( first, x_delta, y_delta );

	/*--- Center the pan zoom into the life field ---*/
	if ( !first )
		{
		zoom_start_x = MULT8( (FIELD_WIDTH  - zoomed_cells_width_nb) / 2 );
		set_zoom_xscrollbar( zoom_start_x );
		zoom_start_y = (FIELD_HEIGHT - zoomed_cells_height_nb) / 2;
		set_zoom_yscrollbar( zoom_start_y );
		}

	/*--- Change the position and the size of the pan rectangle ---*/
	resize_pan();

	/*--- Reset first-time flag ---*/
	first = FALSE;

	return( Pt_CONTINUE );

	}	// zoom_resize
